/*
 * Copyright (C) 2004  Moritz Orbach <irom@apfelboymchen.homeunix.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 */

#include <stdio.h>
#include <errno.h>
#include <stdlib.h>
#include <strings.h>
#include <unistd.h>
#include <getopt.h>

#include "status.h"
#include "options.h"
#include "config.h"



static struct option const long_options[] =
{
	{ "help",      no_argument,       NULL, 'h' },
	{ "version",   no_argument,       NULL, 'v' },
	{ "delay",     required_argument, NULL, 'd' },
	{ "maxsize",   required_argument, NULL, 's' },
	{ "minborder", no_argument,       NULL, 'b' },
	{ "logfile",   no_argument,       NULL, 'l' },
	{ NULL, 0, NULL, 0 }
};


void set_defaults()
{
	/* delay 15 min */
	cfg.delay = 900;
	cfg.maxsize = 150;
	cfg.minborder = 100;
}

/* ------ */

/* unsused */
void cfg_file()
{
	FILE *fp;
	const char *rcfile = "/home/mori/.zufallrc\0";
	char line[82];
	int length = 0;

	char option[80];
	char argument[80];

	fp = fopen(rcfile, "r");
	if (fp == NULL)
		return;

	while (fgets (line + length, 82 - length, fp))
		printf("line: %s", line);
	sscanf(line, "%s = %s\n", option, argument);
	printf("option: %s\nargument: %s\n", option, argument);

	fclose(fp);
	return;
}       
/* ------ */

void version(void)
{
	printf("%s %s\n", PACKAGE_NAME, PACKAGE_VERSION);
	printf("Written by %s\n", AUTHORS);
	exit(EXIT_SUCCESS);
}
 
/* -pedantic: warning: string length `585' is greater than the length `509' ISO C89 compilers are required to support */
void usage(int status)
{
	printf("Usage: %s [OPTION] [COMMAND [ARG]...]\n", program_name);
	printf(
			"  -d, --delay=DELAY          number of seconds an image is shown\n"
			"                             (default 900)\n"
			"  -s  --maxsize=PERCENT      maximum size in percent\n"
			"  -b  --minborder=PIXEL      ignore --maxsize if the remaining border would be\n"
			"                             smaller than --minborder\n"
			"                             (default 100)\n"
			"  -l  --logfile              don't buffer stdout\n"
			"\n"
			"      --help                 display this help and exit\n"
			"      --version              output version information and exit\n"
			"\n"
			"SIGNALS:\n"
			"  HUP                        change background\n"
			"  USR1                       info\n"
			"  TERM                       exit\n"
	      );
	exit (status);
}

int cfg_cmdln(int *argc, char **argv, int *diroffset)
{
	char c;

	program_name = argv[0];

	if (*argc == 1) {
		printf("No image-directory specified.\n");
		exit(EXIT_FAILURE);
	}

	while ((c = getopt_long (*argc, argv, "hvld:s:b:", long_options, NULL)) != -1)
		switch(c)
		{
			case 'h':
				usage(EXIT_SUCCESS);
			break;
			case 'v':
				version();
			break;

			case 'd':
				cfg.delay = atoi(optarg);
				/* nicht schick, aber ich hab kein bock auf verberssern */
				if (cfg.delay < 5) {
					fprintf(stderr, "Delay too small. Setting to 5\n");
					cfg.delay = 5;
				}
			break;
			case 's':
				/* hier muss ich mal funktion schreiben */
				cfg.maxsize = atoi(optarg);
			break;
			case 'b':
				cfg.minborder = atoi(optarg);
			break;
			case 'l':
				/* setlinebuf(stdout);
				 * geht nicht -- "implicit declaration" trotz stdio.h */
				setvbuf(stdout, (char *)NULL, _IOLBF, 0);
			break;

			default:
			usage(EXIT_FAILURE);
		}

	/* ab optindex die brigen parameter */
	/* warum kommt argv nicht im caller an??
	*argc = *argc - optind;
	argv = &argv[optind];
	*/
	*diroffset = optind;
	return(1);
}

