/*
 * Copyright (C) 2004  Moritz Orbach <irom@apfelboymchen.homeunix.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 */

#include <stdio.h>
#include <string.h>
#include "memman.h"

struct s_imagelist {
	struct s_imagelist *pnext;
	char *path;
};

struct s_imagelist *pfirst = NULL;
int count = 0;

/* gibt speicher fr initialisierten neuen knoten zurck */
struct s_imagelist *knot_init(void)
{
	struct s_imagelist *knot;

	/* hier muss der typ angegeben werden, variable geht nicht! ?. */
	knot = getmem(sizeof(struct s_imagelist));
	knot->pnext = NULL;
	knot->path = NULL;

	return(knot);
}

int list_init(void)
{
	extern struct s_imagelist *pfirst;
	struct s_imagelist *imagelist;

	imagelist = knot_init();
	pfirst = imagelist;

	if(pfirst == NULL)
		return(0);
	else
		return(1);
}

/* anzahl der angelegten knoten */
int list_count(void)
{
	extern int count;

	return(count);
}

/* erster knoten hat die nummer 1 */
struct s_imagelist *list_get(int num)
{
	extern struct s_imagelist *pfirst;
	struct s_imagelist *pcurrent;
	int pos = 1;
	int ok = 1;
	int listcount;

	listcount = list_count();

	if (listcount == 0) return(NULL);

	if (num > listcount) {
		fprintf(stderr, "huh? knoten %d? ich hab doch nur %d?!\n", num, listcount);
		ok = 0;
	}

	if (ok) {

		pcurrent = pfirst;
		while (pcurrent->pnext != NULL && pos < num) {
			pcurrent = pcurrent->pnext;
			pos++;
		}
	}

	if (ok)
		return(pcurrent);
	else
		return(NULL);
}

/* todo: returnval */
int list_append(const char *path)
{
	extern int count;
	extern struct s_imagelist *pfirst;

	struct s_imagelist *pcurrent = pfirst;
	struct s_imagelist *pnew;

	if (pcurrent == NULL) {
		printf("HMMMMM........\n");
		return(1);
	}
	/* letzten knoten finden */
	while (pcurrent->pnext != NULL)
		pcurrent = pcurrent->pnext;

	if (pcurrent->path != NULL) {
		/* printf("neuer knoten muss her\n"); */
		pnew = knot_init();
		pcurrent->pnext = pnew;
		pcurrent = pnew;
	}
	pcurrent->path = getmem(strlen(path) + 1);
	strcpy(pcurrent->path, path);
	count++;

	return(0);
}

/* delete one node */
int list_delete(unsigned int pos)
{
	extern struct s_imagelist *pfirst;
	extern int count;
	struct s_imagelist *pdelete, *pprev;
	int listcount, ok = 1;

	listcount = list_count();
	if (pos > listcount) {
		fprintf(stderr, "huh? knoten %d loeschen? ich hab doch nur %d?!\n", pos, listcount);
		ok = 0;
	}

	if (ok && pos > 1) {
		/* vorherigen knoten finden */
		pprev =  list_get(pos-1);
		ok = pprev != NULL;

		/* knoten zum lschen finden */
		if (ok) {
			pdelete = pprev->pnext;
			ok = pdelete != NULL;
		}

		/*
		 * vorherigen und nchsten verbinden, free mem
		 * nchster kann ruhig NULL sein
		 */
		if (ok) {
			pprev->pnext = pdelete->pnext;
			freemem(pdelete->path, strlen(pdelete->path) + 1);
			freemem(pdelete, sizeof(struct s_imagelist));
		}
		count--;
	}
	
	if (ok && pos == 1) {
		/* erster knoten */
		pdelete = list_get(pos);
		ok = pdelete != NULL;

		if (count > 1) {
			if (ok) {
				pfirst = pdelete->pnext;
				freemem(pdelete->path, strlen(pdelete->path) + 1);
				freemem(pdelete, sizeof(struct s_imagelist));
				pdelete->path = NULL;
				count--;
			} else printf("das drfte nu echt nicht passieren!\n");
		} else {
			/* gleichzeitig letzter knoten - nur daten lschen, sonst msste man neu initialisieren */
			freemem(pdelete->path, strlen(pdelete->path) + 1);
			pdelete->path = NULL;
		}
	}

	return(ok);
}

/* delete the list */
int list_destroy(void)
{
	int i;
	int ok = 1;

	for (i = list_count(); i > 0; i--) {
		if (!list_delete(i))
			ok = 0;
		/*
		printf("%d: %d\n", i, ok == 1); 
		fflush(NULL);
		*/
	}

	/* speicher fr ersten knoten lschen */
	freemem(pfirst, sizeof(struct s_imagelist));

	return ok;
}

unsigned int list_allocated(void)
{
	return allocated();
}

