/*
 * Copyright (C) 2004  Moritz Orbach <irom@apfelboymchen.homeunix.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * Parts of this file are ripped from Esetroot.c written by Nat Friedman 
 * <ndf@mit.edu> with modifications by Gerald Britton <gbritton@mit.edu> and 
 * Michael Jennings <mej@eterm.org>
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <errno.h>

#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Xatom.h>
#include <X11/Xos.h>
#include <Imlib2.h>

#include "config.h"

#define Xdepth	 (DefaultDepth(Xdisplay, screen))

void set_pixmap_property(Display *Xdisplay, Window Xroot, Pixmap p)
{
	Atom prop_root;

	prop_root = XInternAtom(Xdisplay, "_XROOTPMAP_ID", True);

	if (prop_root == None)
	{
		/* This will locate the property, creating it if it doesn't exist */
		prop_root = XInternAtom(Xdisplay, "_XROOTPMAP_ID", False);
	}

	/* The call above should have created it.  If that failed, we can't continue. */
	if (prop_root == None) {
		fprintf(stderr, "Esetroot:  creation of pixmap property failed.\n");
		exit(1);
	}
	XChangeProperty(Xdisplay, Xroot, prop_root, XA_PIXMAP, 32, PropModeReplace, (unsigned char *) &p, 1);
}

void eterm_set_pixmap_property(Display *Xdisplay, Window Xroot, Pixmap p)
{
	Atom prop_root, prop_esetroot, type;
	int format;
	unsigned long length, after;
	unsigned char *data_root, *data_esetroot;

	/* A program that wants to set the background should:
	 *  - Create a persistant pixmap by calling to the server with:
	 *        dpy = XOpenDisplay(display_name);
	 *        XSetCloseDownDisplay (dpy, RetainPixmap);
	 *        pmap = XCreatePixmap (dpy, DefaultRootWindow (dpy), width, height,
	 *                              DefaultDepthOfScreen (DefaultScreenOfDisplay (dpy)));
	 *        XCloseDisplay (dpy)
	 *
	 * - Grab the server
	 * - See if ESETROOT_PMAP_ID is set
	 * - If so, call XKillClient(xdisplay, *(Pixmap*)data_esetroot)
	 * - Change ESETROOT_PMAP_ID and _XROOTPMAP_ID to point to the * new pixmap
	 * - Set the pixmap as the background for the root window
	 * - Ungrab the server
	 *
	 * http://mail.gnome.org/archives/wm-spec-list/2002-January/msg00006.html
	 * http://www.eterm.org/docs/view.php?doc=ref
	 */

	/* First check to see if the properties already exist, and if they are equal */
	prop_root = XInternAtom(Xdisplay, "_XROOTPMAP_ID", True);
	prop_esetroot = XInternAtom(Xdisplay, "ESETROOT_PMAP_ID", True);

	if (prop_root != None && prop_esetroot != None) {

		XGetWindowProperty(Xdisplay, Xroot, prop_root, 0L, 1L, False, AnyPropertyType,
				   &type, &format, &length, &after, &data_root);

		if (type == XA_PIXMAP) {
			XGetWindowProperty(Xdisplay, Xroot, prop_esetroot, 0L, 1L, False, AnyPropertyType,
					   &type, &format, &length, &after, &data_esetroot);
			if (data_root && data_esetroot && type == XA_PIXMAP && *((Pixmap *) data_root) == *((Pixmap *) data_esetroot)) {
				/* It's safe.  Kill the pixmap. */
				XKillClient(Xdisplay, *((Pixmap *) data_root));
			}
		}
	}

	/* This will locate the property, creating it if it doesn't exist */
	prop_root = XInternAtom(Xdisplay, "_XROOTPMAP_ID", False);
	prop_esetroot = XInternAtom(Xdisplay, "ESETROOT_PMAP_ID", False);

	/* The call above should have created it.  If that failed, we can't continue. */
	if (prop_root == None || prop_esetroot == None) {

		fprintf(stderr, "Error:  Creation of pixmap property failed.\n");
		exit(1);
	}
	XChangeProperty(Xdisplay, Xroot, prop_root, XA_PIXMAP, 32, PropModeReplace,
			(unsigned char *) &p, 1);
	XChangeProperty(Xdisplay, Xroot, prop_esetroot, XA_PIXMAP, 32, PropModeReplace,
			(unsigned char *) &p, 1);
	/*
	XSetCloseDownMode(Xdisplay, RetainPermanent);
	*/
}

/* bergebene farbe nach hex konvertieren und an color anhngen */
void append_color(int color_dez, char color[7])
{
	/* 2 hex + \0 */
	char temp[3];

	snprintf(temp, 3, "%.2x\n", color_dez);
	strncat(color, temp, 6);
}


unsigned int rnd(unsigned int max)
{
	return(rand() /(int)(((unsigned)RAND_MAX + 1) / max));
}

void randomcolor(int maxval, char color[7])
{
	int rgb;
	unsigned int r;

	color[0] = '#';
	/* ohne weiss strncat anscheinend nicht, wo der string aufhoert */
	color[1] = '\0';

	for (rgb = 0; rgb < 3; rgb++)
	{
		r = rnd(maxval);

		append_color(r, color);
		printf("color: %d\n", r);
	}
	printf("fertig: %s\n", color);
}


int acmp(int *array1, int *array2, unsigned int size)
{
	int i = 0, equal = 1;

	while (i < size && equal)
	{
		equal = array1[i] == array2[i];
		i++;			
	}

	return equal;
}

/* alle 4 ecken angucken, durchschnitt der farben bilden */
/* oder: 2 ecken angucken, nur wenn nicht gleich durchschnitt */
/*
 * lieber doch nur 2 ecken. Sonst gibt's komische farben wenn in einer ecke 
 * z.B. ein dummes logo ist. Auch ausreichend.
 */
void lookupcolor(int w, int h, char color[7])
{
	Imlib_Color color_return;
	int rgb[3][3], border, i, temp;

	color[0] = '#';
	color[1] = '\0';

	/* links oben */
	imlib_image_query_pixel(0, 0, &color_return);
	rgb[0][0] = color_return.red;
	rgb[0][1] = color_return.green;
	rgb[0][2] = color_return.blue;

	/* links unten */
	imlib_image_query_pixel(0, h-1, &color_return);
	rgb[1][0] = color_return.red;
	rgb[1][1] = color_return.green;
	rgb[1][2] = color_return.blue;

#ifdef gibznit
	/* rechts oben */
	imlib_image_query_pixel(w-1, 0, &color_return);
	rgb[2][0] = color_return.red;
	rgb[2][1] = color_return.green;
	rgb[2][2] = color_return.blue;

	/* rechts unten */
	imlib_image_query_pixel(w-1, h-1, &color_return);
	rgb[3][0] = color_return.red;
	rgb[3][1] = color_return.green;
	rgb[3][2] = color_return.blue;


	if (acmp(rgb[0], rgb[1], 3) &&
 	    acmp(rgb[0], rgb[2], 3) &&
	    acmp(rgb[0], rgb[3], 3))
		/* alle 4 ecken gleich */
		/* nicht skalieren */
		*fit = 0;
#endif

	/* durchschnitt der ersten beiden ecken */
	for (i = 0; i < 3; i++) {
		temp = 0;
		for (border = 0; border < 2; border++)
			temp += rgb[border][i];

		temp = temp / 2;

		append_color(temp, color);

	}
}

int setroot(Display *Xdisplay, Window Xroot, Screen *scr, int screen,
		const char *image, const char *bgcolor)
{
	unsigned int scale = 0, center = 0, fit = 1;
	double maxsize = cfg.maxsize / 100;
	int minborder = cfg.minborder;
	Imlib_Image im;
	Pixmap p = None, temp_pmap = None, m = None;
	GC gc;
	XGCValues gcv;
	XColor xcolor;
	int w, h, x, y;
	char color[7];

	/* image laden */
	im = imlib_load_image_immediately(image);
	if (im == NULL) {
		fprintf(stderr, "Unable to load image file \"%s\".\n", image);
		return(1);
	}
	imlib_context_set_image(im);

	/* gre festlegen */
	/*
	if (scale) {
		center = 0;
		mirror = 0;
	}

	if (scale) {
		w = scr->width;
		h = scr->height;
	} else if (mirror) {
		w = imlib_image_get_width() * 2;
		h = imlib_image_get_height() * 2;
	} else {
	*/
	w = imlib_image_get_width();
	h = imlib_image_get_height();

	/* randfarbe */
	if (!strcmp(bgcolor, "")) {
		/*
		randomcolor(70, color);
		nice_randomcolor(color);
		*/
		lookupcolor(w, h, color);

	}
	else
		/* uebergebenes */
		strncpy(color, bgcolor, 7);

	if (fit) {
		double x_ratio, y_ratio;
		int *border, *full;

		x_ratio = ((double) scr->width) / ((double) w);
		y_ratio = ((double) scr->height) / ((double) h);
		/* nicht zerren, kleinere ratio nehmen sonst laeuft's ueber */
		if (x_ratio > y_ratio)
		{
			/* hochkant */
			x_ratio = y_ratio;
			/* rand an dem frei bleibt */
			border = &h;
			full = &scr->height;
		} else {
			/* quer */
			/* rand an dem frei bleibt */
			border = &w;
			full = &scr->width;
		}


		/* nicht mehr als maxsize vergroessern */
		if ((int) (*border * x_ratio) > (int) *border * maxsize)
			x_ratio = maxsize;

		/* trotz maxsize keine rnder kleiner als minborder pixel */
		if (*full - *border * x_ratio < minborder)
			x_ratio = ((double) *full) / ((double) *border);

		w = (int) (w * x_ratio);
		h = (int) (h * x_ratio);
	}



	p = XCreatePixmap(Xdisplay, Xroot, scr->width, scr->height, Xdepth);
	gcv.foreground = gcv.background = BlackPixel(Xdisplay, screen);
	/* broken pipe */
	if (color && XParseColor(Xdisplay, DefaultColormap(Xdisplay, screen), color, &xcolor)
			&& XAllocColor(Xdisplay, DefaultColormap(Xdisplay, screen), &xcolor)) {
		gcv.foreground = gcv.background = xcolor.pixel;
	}
	/* Create a "Graphics Context" TODO: fehler abfangen */
	gc = XCreateGC(Xdisplay, p, (GCForeground | GCBackground), &gcv);

	if (scale) {
		XFillRectangle(Xdisplay, p, gc, 0, 0, w, h);
	}
	if (center || fit) {
		XFillRectangle(Xdisplay, p, gc, 0, 0, scr->width, scr->height);
		x = (scr->width - w) >> 1;
		y = (scr->height - h) >> 1;
	} else {
		x = 0;
		y = 0;
	}

	imlib_context_set_drawable(Xroot);
	imlib_render_pixmaps_for_whole_image_at_size(&temp_pmap, &m, w, h);

	if (temp_pmap != None) {
		if (m) {
			XFreePixmap(Xdisplay, m);
			m = None;
		}
		XSetTile(Xdisplay, gc, temp_pmap);
		XSetTSOrigin(Xdisplay, gc, x, y);
		XSetFillStyle(Xdisplay, gc, FillTiled);
		if (center || fit) {
			XFillRectangle(Xdisplay, p, gc, x, y, w, h);
		} else {
			XFillRectangle(Xdisplay, p, gc, x, y, scr->width, scr->height);
		}

		/* kommentar aus control-center (applier.c)
		 * Set the root pixmap, and properties pointing to it. We
		 * do this atomically with XGrabServer to make sure that
		 * we won't leak the pixmap if somebody else it setting
		 * it at the same time. (This assumes that they follow the
		 * same conventions we do)
		 */
		XGrabServer(Xdisplay);
		set_pixmap_property(Xdisplay, Xroot, p);
		/*set_pixmap_property(Xdisplay, Xroot, p);*/
		XSetWindowBackgroundPixmap(Xdisplay, Xroot, p);
		XClearWindow(Xdisplay, Xroot);

		XUngrabServer(Xdisplay);
		/* send the request to the server */
		XFlush(Xdisplay);
		XFreePixmap(Xdisplay, temp_pmap);
	}
	XFreeGC(Xdisplay, gc);
	XFreePixmap(Xdisplay, p);
	imlib_free_image();
	return(0);
}
