/*
 * Copyright (C) 2004  Moritz Orbach <irom@apfelboymchen.homeunix.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 */

#include <stdio.h>
#include <sys/types.h>
#include <dirent.h>
#include <unistd.h>
#include <sys/stat.h>
#include <string.h>
#include <stdlib.h>
#include <time.h>
#include <signal.h>

#include <X11/Xutil.h>
#include <Imlib2.h>

#include "imglist.h"
#include "setroot.h"
#include "config.h"
#include "status.h"

/* mtrace() */
#ifdef DEBUG
#include <mcheck.h>
#endif

Display *Xdisplay;
Screen *scr;
Window Xroot;
int screen;

/* vielleicht
 * - links beim rekursieren beachten
 * - vernnftige hintergrundfarbe:
 *   . zufall ohne grelle farben
 *   . r, g und b max- bzw. minimalwerte festlegen
 * - datei auslesen in denen ausnahmen zu default optionen stehen (pro bild). ugh.
 */

int addfiles(const char *startdir)
{
	/* extern */

	/* intern */
	DIR *dirp;
	struct dirent *dp;
	struct stat estat;
	/* warning: ISO C90 forbids variable-size array `directory' */
	char directory[strlen(startdir)];

	int sr; /* stat return */

	strcpy(directory, startdir);

	/* /etc reicht schon fr "Too many open files" :? */
	dirp = opendir(directory);
	if (dirp == NULL) {
		perror(directory);
		return(1);
	}

	while (running() && (dp = readdir (dirp)) != NULL) {


		if (!strcmp(dp->d_name, ".") || !strcmp(dp->d_name, "..")) continue;

		/* verzeichnis + dateien zusammenkopieren (2 ISO C90-fehler) */
		char path[strlen(directory) + strlen(dp->d_name) + 2]; /* +1 \0, +1 '/'*/
		strcpy(path, directory);
		/* nur slash anhngen wenn keiner bergeben wurde */
		if (path[strlen(directory) - 1] != '/') strncat(path, "/", 1);
		strcat(path, dp->d_name);
		/* */

		sr = stat(path, &estat);
		if (sr) {
			perror(path);
			continue;
		}

		if (!S_ISDIR(estat.st_mode))
		{
			/* file */
			list_append(path);
		} else {
			/* verzeichnis */
			addfiles(path);
		}
	}
	closedir(dirp);
	return(0);
}

unsigned int getrandom(unsigned int max)
{
	if (max > 0)
		return(rand() % max + 1);
	else
		return(0);
} 

void exit_func(void)
{

	if (Xdisplay) XCloseDisplay(Xdisplay);
	/* eigentlich || list_allocated != 0, aber das liefert immer peak zurck. k.A.! */
	if (!list_destroy())
		fprintf(stderr, "error freeing memory..\n");
}

void initX(void)
{
	/* clients ohne windows tauchen anscheinend nicht in xlsclients auf */
	Xdisplay = XOpenDisplay(NULL);
	if (!Xdisplay) {
		fprintf(stderr, "%s: Unable to open display\n", PACKAGE_NAME);
		exit(1);
	}

	if (atexit(exit_func)) {
		fprintf(stderr,"zufall: atexit() failed\n");
		exit(2);
	}

	screen = DefaultScreen(Xdisplay);
	Xroot = RootWindow(Xdisplay, screen);
	scr = ScreenOfDisplay(Xdisplay, screen);
	imlib_context_set_display(Xdisplay);
	imlib_context_set_visual(DefaultVisual(Xdisplay, DefaultScreen(Xdisplay)));
	imlib_context_set_anti_alias(1);
	imlib_context_set_dither(1);
	imlib_context_set_blend(1);
}

/* signal handlers */

void infotrap(p)
{
	unsigned int tmp;
	tmp = list_allocated();

	/*
	 * "calling library functions from a signal handler is not guaranteed to work 
	 * in all implementations....."
	 * http://publications.gbdirect.co.uk/c_book/chapter9/signal_handling.html
	 * http://www.delorie.com/gnu/docs/glibc/libc_488.html
	 * hm..
	 */
	/* warning: ISO C90 does not support the `%lf' printf format */
	printf("allocated: %d b (%.2lf kb), %d nodes\n", tmp, (double)tmp / 1024, list_count());
	signal(SIGUSR1, infotrap);
	break_sleep(0);
}

void nop(p)
{
	/* NOP */
	signal(SIGHUP, nop);
	break_sleep(1);
}

void signal_quit()
{
	printf("%s: exiting\n", PACKAGE_NAME);
	quit();
	break_sleep(1);
}

/* --------- */

int main(int argc, char *argv[])
{
	struct s_imagelist *image;
	unsigned int zufall;
	unsigned int tr;	/* time remaining */
	int error = 0;		/* return code */
	int diroffset = 0;	/* grr :( */

	/* killall / CTRL-C */
	signal(SIGTERM, signal_quit);
	signal(SIGINT, signal_quit);

#ifdef DEBUG
	mtrace();
#endif
	set_defaults();
	/* cfg_file(); */
	cfg_cmdln(&argc, argv, &diroffset);
	argc = argc - diroffset;
	argv = &argv[diroffset];

	if (running() && !error) {
		initX();
		list_init();
		srand(time (0));

		int i;
		for (i = 0; i < argc; i++)
			addfiles(argv[i]);
	}
	/* imagechange, info */
	signal(SIGHUP, nop);
	signal(SIGUSR1, infotrap);

	while (running() && !error) {

		zufall = getrandom(list_count());
		image = list_get(zufall);
		if (image == NULL) {
			printf("no pictures left\n");
			break;
		}
		printf("so ein zufall!: %s\n", image->path);

		tr = cfg.delay;
		if (!setroot(Xdisplay, Xroot, scr, screen, image->path, "")) {
			while (tr != 0) {
				/* hintergrund bewundern */
				tr = sleep(tr);
				/* weiterschlafen oder neu schlafen? */
				if (sleep_broken()) {
					/* reset fr bildwechsel oder ende */
					tr = 0;
					break_sleep(0);
				} else {
					/* bildwechsel / signal */
					if (tr != 0) {
						printf("%s\n", image->path);
						printf("%d seconds (%.1d:%.2d) remaining\n", tr, (int)tr / 60, (int)tr % 60);
					}
				}
					
			}
		} else {
			printf("Could not load image, trying next.\n");
			if (!list_delete(zufall)) {
				fprintf(stderr, "error freeing memory..\n");
				error++;
			}
		}
#ifdef DEBUG
		/* komplette liste ausgeben */
		int i;
		struct s_imagelist *descht;

		for (i = 1; i <= list_count(); i++) {
			descht = list_get(i);
			printf("%.2d: %s\n", i, descht->path);

		}
#endif
	}

#ifdef DEBUG
	muntrace();
#endif
	exit(error);
}
